
import unohelper

from com.sun.star.ui import XSidebarPanel, LayoutSize

def Initialize(element, parent):
    """ Initialize panel element.
    
        @param element instance that implements css.ui.XUIElement, 
                        XToolPanel and XSidebarPanel. 
                        Additionally css.container.XNameContainer that 
                        allows you to set XSidebarPanel.
        @param parent parent window for the panel window
        @return child window
    """
    # You can get access to XUIElement interface that keeps document frame and 
    # resource URL of the panel. See IDL reference.
    obj = SidebarUIElementForMacros(
                    XSCRIPTCONTEXT.getComponentContext(), element.Frame, parent)
    # If your window has complicated arrangement, insert css.ui.XSidebarPanel 
    # interface with XSidebarPanel name into the first argument of the initialization function.
    # If this value is not specified, the element returns css.ui.Layout(0, -1, 0) and that 
    # is enough for the panel monopolizes the deck.
    element.insertByName("XSidebarPanel", obj)
    return obj.get_window()


from com.sun.star.awt import XActionListener, XFocusListener, XKeyHandler
from com.sun.star.awt.Key import RETURN

class SidebarUIElementForMacros(unohelper.Base, XSidebarPanel):
    """ Example sidebar element for macros. """
    
    # This dialog should have CommandButton1 and TextField1
    DIALOG_URL = "$(user)/basic/Standard/Dialog4.xdl"
    
    def __init__(self, ctx, frame, parent):
        self.ctx = ctx
        self.frame = frame
        self.parent = parent
        self.window = self._create_window(self.DIALOG_URL)
        self._init_window()
        self._edit_key_handler = None
        self._init_handlers()
    
    # XSidebarPanel
    def getHeightForWidth(self, width):
        return LayoutSize(0, -1, 0)
    
    def get_window(self):
        return self.window
    
    def _create(self, name):
        return self.ctx.getServiceManager().createInstanceWithContext(name, self.ctx)
    
    def _create_window(self, uri, make_visible=True):
        uri = self._create("com.sun.star.util.PathSubstitution").\
                        substituteVariables(uri, True)
        window = self._create("com.sun.star.awt.ContainerWindowProvider").\
                        createContainerWindow(uri, "", self.parent, None)
        if make_visible: window.setVisible(True)
        return window
    
    class ListenerBase(unohelper.Base):
        def __init__(self, act):
            self.act = act
        def disposing(self, ev):
            self.act = None
    
    class ButtonListener(ListenerBase, XActionListener):
        def actionPerformed(self, ev):
            self.act.button_pushed()
    
    def button_pushed(self):
        s = self.window.getControl("TextField1").getText()
        if s:
            self.frame.getController().getModel().getText().getEnd().setString(s)
    
    class FocusListener(ListenerBase, XFocusListener):
        def focusGained(self, ev):
            self.act.edit_focuse_gained()
        def focusLost(self, ev):
            self.act.edit_focus_lost()
    
    def edit_focuse_gained(self):
        self.frame.getContainerWindow().getToolkit().addKeyHandler(self._edit_key_handler)
    
    def edit_focus_lost(self):
        self.frame.getContainerWindow().getToolkit().removeKeyHandler(self._edit_key_handler)
    
    def _init_window(self):
        self.window.getControl("CommandButton1").\
                addActionListener(self.ButtonListener(self))
        self.window.getControl("TextField1").\
                addFocusListener(self.FocusListener(self))
    
    class KeyHandler(ListenerBase, XKeyHandler):
        """   When you push Return key inside liner text field placed in 
            the panel of the sidebar, you can see the key event happens 
            on the document. All key events not consumed by the control 
            implementation will be happen on the document also.
            This behaviour can not be avoided using key listener, 
            because it can not consume key events.
              So we need key handler but to set key handler through  
            css.awt.XUserInputInterception interface of the document 
            controller is not usable (it worked on taskbar).
              Remained way is to add key handler through css.awt.XExtendedToolkit 
            interface to css.awt.Toolkit (the interface is deprecated!). 
              To add key handler to the Toolkit is not good way, 
            most of all key events happen on the office UI are come. 
            In this example, the handler is set by focus gained event and 
            removed by focus lost event of the text field.
            
            This problem happens on the different kind of controlls also.
            Maybe you have to remove keyhandler during the element disposing.
        """
        def keyReleased(self, ev): return True
        def keyPressed(self, ev):
            if ev.KeyCode == RETURN:
                self.act.button_pushed()
                return True
            return False # following handlers takes events
    
    def _init_handlers(self):
        self._edit_key_handler = self.KeyHandler(self)
